#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import socket
import os, os.path

from Decoding_Functions import *
from Scroll_Decoding_Functions import *

# Constant declaration ---------------------------------------

# Allow to launch the codec as pseudo daemon thanks to socket
IS_DAEMON = True
END_OF_SERVICE_CMD = 'END'
SOCKET_ADRESS = "/tmp/tst_socket"
MAX_DATA_LENGTH = 1024
WANTED_FORMAT = 'json'
DEBUG_ON = True
# ------------------------------------------------------------

# Remove the socket file if it already exists
if os.path.exists( SOCKET_ADRESS ):
  os.remove( SOCKET_ADRESS ) 

# Creation of the socket and bind the socket to the port
CommSocket = socket.socket( socket.AF_UNIX, socket.SOCK_STREAM )
CommSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
CommSocket.bind(SOCKET_ADRESS)

# Listen for incoming connections
CommSocket.listen(1)

InConnection = None

try:

	if DEBUG_ON:
		print("Launching Service")

	while( IS_DAEMON ):

		# Wait for a connection
		if DEBUG_ON:
				print("Wait for connection")
		InConnection, ClientAddr = CommSocket.accept()
	
		if DEBUG_ON:
			print("Connection from :", ClientAddr)
		
		DataToProcess = InConnection.recv( MAX_DATA_LENGTH )

		if ( DataToProcess == END_OF_SERVICE_CMD ):

			# The other service asks the end of this service : break out of the while loop
			if DEBUG_ON:
				print("End of Service")
		
			# Clean up the connection
			InConnection.close()
			IS_DAEMON = False

		else:
			if DEBUG_ON:
				print ("New data to decode :", DataToProcess)
		
			# We received a frame from the other service to decode
			DecodedData = Decoding_JSON(DataToProcess, False)
	
			if DEBUG_ON:
				print ("Sending decoded data :", DecodedData, "...")
	
			# Send the decoded data
			InConnection.sendall(DecodedData.encode())

except KeyboardInterrupt:

	if InConnection is not None:
		# Clean up the connection
		InConnection.close()
	
	# Close the socket
	if DEBUG_ON:
		print("Closing socket...")
	CommSocket.close()
	
	# Delete the socket file
	os.remove( SOCKET_ADRESS )
	
except:

	if InConnection is not None:
		# Clean up the connection
		InConnection.close()
	
	# Close the socket
	if DEBUG_ON:
		print("Closing socket...")
	CommSocket.close()
	
	# Delete the socket file
	os.remove( SOCKET_ADRESS )

else:
	# Close the socket
	if DEBUG_ON:
		print("Closing socket...")
	CommSocket.close()

	# Delete the socket file
	os.remove( SOCKET_ADRESS )

